#include "TrafficLight.h"

TrafficLight::TrafficLight(
                Led &redLed, Led &yellowLed, Led &greenLed,
                LedBlinker &greenLedBlinker, PushButton &button)
{
  _redLed = redLed;
  _yellowLed = yellowLed;
  _greenLed = greenLed;
  _greenLedBlinker = greenLedBlinker;
  _button = button;
}

void TrafficLight::init()
{
  _redLed.init();	// aanroepen init functie in de verschillende Child classen
  _yellowLed.init();
  _greenLed.init();
  _button.init();
  delay(100);

  red();		//startsituatie – rode led aan. 
}

void TrafficLight::red()
{
  _redLed.on();
  _yellowLed.off();
  _greenLed.off();
}

void TrafficLight::yellow()
{
  _redLed.off();
  _yellowLed.on();
  _greenLed.off();  
}

void TrafficLight::green()
{
  _redLed.off();
  _yellowLed.off();
  _greenLed.on();  
}

void TrafficLight::allLedsOff()
{
  _redLed.off();
  _yellowLed.off();
  _greenLed.off();  
}

void TrafficLight::update()
{
  unsigned long timeNow = millis();

  switch (state)
  {
    case STATE_RED:
    {
      red();
      if (_button.isPressed())
      {
        state = STATE_YELLOW;
        yellowStateStartTime = timeNow;
      }
      break;
    }
    
    case STATE_YELLOW:
    {
      yellow();
      if (timeNow-yellowStateStartTime > yellowStateDuration)
      {
        state = STATE_GREEN;
        greenStateStartTime = timeNow;
      }
      break;
    }

    case STATE_GREEN:
    {
      green();
      if (timeNow-greenStateStartTime > greenStateDuration)
      {
        state = STATE_BLINK_GREEN;
        allLedsOff();
        blinkGreenStateStartTime = timeNow;
      }
      break;
    }
    
    case STATE_BLINK_GREEN:
    {
      _greenLedBlinker.update();
      if (timeNow-blinkGreenStateStartTime > blinkGreenStateDuration)
      {
        state = STATE_RED;
      }
      break;
    }
    default:
    {
      state = STATE_RED;
    }
  }
}

